@@ -1,4 +1,4 @@
-__version__ = "2.1.0"
+__version__ = "2.1.1"
 __author__ = "Hanjin Liu"
 __email__ = "liuhanjin-sc@g.ecc.u-tokyo.ac.jp"
 
@@ -18,13 +18,14 @@ from .axes import slicer
 r"""
 Inheritance
 -----------
-        np.ndarray _    _ AxesMixin _
-                    \  /             \
-            ____ MetaArray _     LazyImgArray
-          /        /        \ 
-  LabeledArray  Label    PropArray
-    /     \
-ImgArray PhaseArray
+
+AxesMixin -----> LazyImgArray
+    |
+    +-----> MetaArray --+--> LabeledArray --+--> ImgArray
+    |                   |                   |
+np.ndarray              +--> Label          +--> PhaseArray
+                        |
+                        +--> PropArray
 
 """
 
@@ -36,9 +37,9 @@ del logging
 
 # dtypes
 from numpy import (
-	uint8, uint16, uint32, uint64,
-	int8, int16, int32, int64,
-	float16, float32, float64,
-	complex64, complex128,
-	bool_,
+    uint8, uint16, uint32, uint64,
+    int8, int16, int32, int64,
+    float16, float32, float64,
+    complex64, complex128,
+    bool_,
 )
