@@ -1,12 +1,10 @@
 from __future__ import annotations
-import warnings
 import numpy as np
-from numpy.typing import ArrayLike, DTypeLike
 import os
 from pathlib import Path
 import itertools
 from functools import partial
-from typing import TYPE_CHECKING, Any, Sequence
+from typing import TYPE_CHECKING, Any, Sequence, MutableMapping, Protocol
 from warnings import warn
 from scipy import ndimage as ndi
 
@@ -21,19 +19,130 @@ from ..utils.axesop import complement_axes, find_first_appeared
 from ..utils.deco import check_input_and_output, dims_to_spatial_axes, same_dtype
 from ..io import imsave
 from ..collections import DataList, DataDict
-from ..axes import ImageAxesError, AxesLike, slicer
+from ..axes import ImageAxesError, AxesLike, slicer, Axes
 from .._types import Dims, nDInt, nDFloat, Callable, Coords, Iterable, PaddingMode
 from ..array_api import xp
 
 if TYPE_CHECKING:
     from typing_extensions import Self
     from ..frame import PathFrame
+    from numpy.typing import ArrayLike, DTypeLike
+    from ..roi import RoiList
+
+
+class SupportAxesSlicing(Protocol):
+    """A protocol that covariate objects must follow."""
+
+    @property
+    def axes(self) -> Axes:
+        """Axes object bound to the object."""
+    
+    def _dimension_matches(self, array: MetaArray) -> bool:
+        """Check if self matches array's shape and axes."""
+    
+    def copy(self) -> Self:
+        """Shallow copy of the object."""
+    
+    def __getitem__(self, key) -> SupportAxesSlicing | None:
+        """Slice object."""
+    
+    def _slice_by(self, key) -> SupportAxesSlicing | None:
+        """Slice object."""
+
+def _fmt_slice(k, length: int):
+    if isinstance(k, slice):
+        return slice(*k.indices(length))
+    else:
+        return k
+
+class ArrayCovariates(MutableMapping[str, SupportAxesSlicing]):
+    """A dictionary of covariate objects for a MetaArray."""
+
+    def __init__(self, data: dict[str, SupportAxesSlicing], parent: MetaArray):
+        import weakref
+        self._data = data
+        self._parent_ref = weakref.ref(parent)
+    
+    @property
+    def parent(self):
+        """Return the parent MetaArray object"""
+        out = self._parent_ref()
+        if out is None:
+            raise ValueError("Parent LabeledArray is deleted.")
+        return out
+
+    def construct_by_copying(self, parent: MetaArray | None) -> Self:
+        if parent is None:
+            parent = self.parent
+        data: dict[str, SupportAxesSlicing] = {}
+        for k, value in self.items():
+            if parent.axes.contains(value.axes):
+                data[k] = value.copy()
+        
+        return self.__class__(data, parent)
+    
+    def construct_by_slicing(self, key, next_parent: MetaArray | None) -> Self:
+        parent = self.parent
+        if next_parent is None:
+            next_parent = parent
+        
+        data: dict[str, SupportAxesSlicing] = {}
+        for k, value in self.items():
+            if value is not None:
+                if not isinstance(key, np.ndarray):
+                    if isinstance(key, tuple):
+                        _keys = key
+                    else:
+                        _keys = (key,)
+                    label_sl = tuple(
+                        _fmt_slice(_keys[i], parent.shape[i])
+                        for i, a in enumerate(parent.axes) 
+                        if (a in value.axes and i < len(_keys))
+                    )
+                            
+                    if len(label_sl) == 0 or len(label_sl) > len(value.axes):
+                        label_sl = ()
+                else:
+                    label_sl = key
+
+                if hasattr(value, "_slice_by"):
+                    data[k] = value._slice_by(label_sl)
+                else:
+                    data[k] = value[label_sl]
+
+        return self.__class__(data, next_parent)
+
+    def __getitem__(self, key: str) -> SupportAxesSlicing:
+        return self._data[key]
+    
+    def __setitem__(self, key: str, value: SupportAxesSlicing) -> None:
+        if value is None:
+            self.pop(key, None)
+            return
+
+        parent = self.parent
+        if not value._dimension_matches(parent):
+            raise ValueError(
+                f"Shape of input object ({value!r}) does not match the "
+                f"parent array ({parent.shape_info})."
+            )
+        self._data[key] = value
+    
+    def __delitem__(self, key: str) -> None:
+        del self._data[key]
+    
+    def __len__(self) -> int:
+        return len(self._data)
+
+    def __iter__(self):
+        return iter(self._data)
+    
 
 class LabeledArray(MetaArray):
     _name: str
     _source: Path | None
     _metadata: dict[str, Any]
-    _labels: Label | None
+    _covariates: ArrayCovariates
     
     def __new__(
         cls: type[LabeledArray], 
@@ -47,23 +156,43 @@ class LabeledArray(MetaArray):
         self: LabeledArray = super().__new__(
             cls, obj, name, axes, source, metadata, dtype
         )
-        self._labels = None
+        self._covariates = ArrayCovariates({}, self)
+        
         return self
+    
+    @MetaArray.axes.setter
+    def axes(self, value: AxesLike):
+        if not hasattr(self, "_axes"):
+            # not initialized yet
+            MetaArray.axes.fset(self, value)
+            return
+        old_axes = self.axes
+        MetaArray.axes.fset(self, value)
+        new_axes = self.axes
+        _old_to_new_map = {k: v for k, v in zip(old_axes, new_axes)}
+
+        for v in self._covariates.values():
+            v.axes = [_old_to_new_map[a] for a in v.axes]
 
     @property
     def range(self) -> tuple[float, float]:
         """Return min/max range of the array."""
         return self.min(), self.max()
     
+    @property
+    def covariates(self) -> ArrayCovariates:
+        """Get all the covariates."""
+        return self._covariates
+
     @property
     def labels(self) -> Label | None:
         """The label of the image."""
-        return self._labels
+        return self.covariates.get("labels")
     
     @labels.setter
     def labels(self, value: np.ndarray | None):
         if value is None:
-            self._labels = None
+            self.covariates.pop("labels", None)
             return
 
         if value is self:
@@ -81,34 +210,58 @@ class LabeledArray(MetaArray):
             axes = str(self.axes)[-arr.ndim:]
             value = Label(arr, axes=axes).optimize()
         
-        
-        if not _shape_match(self, value):
+        if not value._dimension_matches(self):
             raise ValueError(
                 f"Shape of input label ({value.shape_info}) does not match the "
                 f"parent array ({self.shape_info})."
             )
-        self._labels = value
+        self.covariates["labels"] = value
     
     @labels.deleter
     def labels(self):
-        self._labels = None
+        self.covariates.pop("labels", None)
+    
+    @property
+    def rois(self) -> RoiList:
+        """ROIs of the image."""
+        return self.covariates.get("rois")
+    
+    @rois.setter
+    def rois(self, val) -> None:
+        from ..roi import RoiList, POS
+        if isinstance(val, RoiList):
+            if val.axes[0] == POS and val.axes[1:] == self.axes[1:]:
+                import copy
+                val = copy.copy(val)
+                val.axes = self.axes[0] + val.axes[1:]
+            
+            self.covariates["rois"] = val
+        else:
+            self.covariates["rois"] = RoiList(self.axes, val)
+    
+    @rois.deleter
+    def rois(self) -> None:
+        self.covariates.pop("rois", None)
     
     def set_scale(self, other=None, **kwargs) -> None:
         super().set_scale(other, **kwargs)
-        if self.labels is not None:
-            self.labels.set_scale(other, **kwargs)
+        for cov in self.covariates.values():
+            if hasattr(cov, "set_scale"):
+                cov.set_scale(other, **kwargs)
         return None
         
         
     def _repr_dict_(self):
         if self.labels is not None:
             labels_shape_info = self.labels.shape_info
+            if len(labels_shape_info) == 0:
+                labels_shape_info = "scalar"
         else:
             labels_shape_info = "No label"
         return {
             "name": self.name,
             "shape": self.shape_info,
-            "label shape ": labels_shape_info,
+            "label shape": labels_shape_info,
             "dtype": self.dtype,
             "source": self.source,
             "scale": self.scale,
@@ -166,47 +319,27 @@ class LabeledArray(MetaArray):
     # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
     
     def __array_finalize__(self, obj):
-        self._labels = getattr(self, "_labels", None)
         super().__array_finalize__(obj)
-        if isinstance(obj, LabeledArray):
-            self._view_labels(obj)
+        self._inherit_covariates(obj)
     
     def _set_info(self, other: Self, new_axes: Any = MetaArray._INHERIT):
-        self._labels = getattr(self, "_labels", None)
+        super()._set_info(other, new_axes)
+        self._inherit_covariates(other)
+    
+    def _inherit_covariates(self, other: Self):
+        if isinstance(other, LabeledArray):
+            if other is not self:
+                self._covariates = other.covariates.construct_by_copying(self)
+            else:
+                self._covariates = other._covariates
+        else:
+            self._covariates = ArrayCovariates({}, self)
+    
+    def _getitem_additional_set_info(self, other: Self, key, new_axes):
+        self._covariates = getattr(self, "covariates", ArrayCovariates({}, self))
         super()._set_info(other, new_axes)
         if isinstance(other, LabeledArray):
-            self._view_labels(other)
-    
-    def _view_labels(self, other: Self):
-        """Make a view of label **if possible**."""
-        if (
-            other.labels is not None and
-            self.axes.contains(other.labels.axes) and
-            _shape_match(self, other.labels)
-        ):
-            if self is not other:
-                self.labels = other.labels.copy()
-            else:
-                self.labels = other.labels
-        
-    
-    def _getitem_additional_set_info(self, other: Self, **kwargs):
-        super()._getitem_additional_set_info(other, **kwargs)
-        key = kwargs["key"]
-        if other.axes and other.labels is not None and not isinstance(key, np.ndarray):
-            if isinstance(key, tuple):
-                _keys = key
-            else:
-                _keys = (key,)
-            label_sl = [_keys[i] for i, a in enumerate(other.axes) 
-                        if a in other.labels.axes and i < len(_keys)]
-                    
-            if len(label_sl) == 0 or len(label_sl) > other.labels.ndim:
-                label_sl = (slice(None),)
-            try:
-                self.labels = other.labels[tuple(label_sl)]
-            except IndexError as e:
-                warn(f"Labels was not inherited due to IndexError : {e}", UserWarning)
+            self._covariates = other.covariates.construct_by_slicing(key, self)
         
         return None
     
@@ -303,57 +436,10 @@ class LabeledArray(MetaArray):
         return None
 
     @dims_to_spatial_axes
-    def imshow(self, label: bool = False, dims = 2, alpha: float = 0.3, **kwargs):
-        from ._utils import _plot as _plt
-        if label and self.labels is None:
-            label = False
-        if self.ndim == 1:
-            _plt.plot_1d(self.value, **kwargs)
-        elif self.ndim == 2:
-            if label:
-                _plt.plot_2d_label(self.value, self.labels.value, alpha, **kwargs)
-            else:
-                _plt.plot_2d(self.value, **kwargs)
-            self.hist()
-            
-        elif self.ndim == 3:
-            if "c" not in self.axes:
-                imglist = self.split(axis=find_first_appeared(self.axes, include=self.axes, exclude=dims))
-                if len(imglist) > 24:
-                    warnings.warn(
-                        "Too many images. First 24 images are shown.",
-                        UserWarning,
-                    )
-                    imglist = imglist[:24]
-                if label:
-                    _plt.plot_3d_label(imglist.value, imglist.labels.value, alpha, **kwargs)
-                else:
-                    _plt.plot_3d(imglist, **kwargs)
+    def imshow(self, label: bool = False, dims = 2, plugin="matplotlib", **kwargs):
+        from ._imshow import imshow
+        return imshow(self, label=label, dims=dims, plugin=plugin, **kwargs)
 
-            else:
-                n_chn = self.shape.c
-                fig, ax = _plt.subplots(1, n_chn, figsize=(4*n_chn, 4))
-                for i in range(n_chn):
-                    img = self[f"c={i}"]
-                    if label:
-                        _plt.plot_2d_label(img.value, img.labels.value, alpha, ax[i], **kwargs)
-                    else:
-                        _plt.plot_2d(img.value, ax=ax[i], **kwargs)
-        else:
-            raise ValueError("Image must have three or less dimensions.")
-        
-        _plt.show()
-
-        return self
-
-    def imshow_comparewith(self, other: Self, **kwargs):
-        from ._utils import _plot as _plt
-        fig, ax = _plt.subplots(1, 2, figsize=(8, 4))
-        _plt.plot_2d(self.value, ax=ax[0], **kwargs)
-        _plt.plot_2d(other.value, ax=ax[1], **kwargs)        
-        _plt.show()
-        return self
-    
     # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
     #   Interpolation
     # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #    
@@ -378,9 +464,7 @@ class LabeledArray(MetaArray):
         ----------
         coordinates : ArrayLike
             Interpolation coordinates. Must be (D, N) or (D, X_1, ..., X_D) shape.
-        {mode}
-        {cval}
-        {order}
+        {mode}{cval}{order}
         prefilter : bool, optional
             Spline prefilter applied to the array. By default set to True if ``order`` is larger
             than 1.
@@ -632,10 +716,7 @@ class LabeledArray(MetaArray):
         
         Parameters
         ----------
-        {order}
-        {mode}
-        {dims}
-        {update}
+        {order}{mode}{dims}{update}
             
         Returns
         -------
@@ -899,88 +980,21 @@ class LabeledArray(MetaArray):
     
     @_docs.write_docs
     @dims_to_spatial_axes
-    @check_input_and_output
     def label(
         self,
         ref_image: np.ndarray | None = None,
+        filt: Callable[..., bool] | None = None,
         *,
         dims: Dims = None,
         connectivity: int | None = None
     ) -> Label:
         """
         Label image using skimage's label().
-
-        Parameters
-        ----------
-        ref_image : array, optional
-            Image to make label, by default self is used.
-        {dims}
-        {connectivity}
-
-        Returns
-        -------
-        Label
-            Labeled image.
         
-        Examples
-        --------
-        Label the image with threshold and visualize with napari.
-        
-            >>> thr = img.threshold()
-            >>> img.label(thr)
-            >>> ip.gui.add(img)
-        """        
-        # check the shape of label_image
-        if ref_image is None:
-            ref_image = self
-        else:
-            if not isinstance(ref_image, MetaArray):
-                ref_image = MetaArray(
-                    np.asarray(ref_image),
-                    axes=self.axes[-self.ndim:]
-                )
-            if not self.axes.contains(ref_image.axes):
-                raise ImageAxesError(
-                    "Not all the axes in `ref_image` are included in self: "
-                    f"{ref_image.axes} and {self.axes}"
-                )
-            elif not _shape_match(self, ref_image):
-                raise ImageAxesError("Shape mismatch.")
-        
-        c_axes = complement_axes(dims, self.axes)
-        labels = largest_zeros(ref_image.shape)
-        
-        labels[:] = ref_image._apply_dask(
-            skmes.label, 
-            c_axes=c_axes, 
-            kwargs=dict(background=0, connectivity=connectivity)
-        ).view(np.ndarray)
-    
-        # correct the label numbers of `labels`
-        labels = labels.view(Label)
-        labels._set_info(ref_image)
-        labels = labels.increment_iter(c_axes).optimize()
-        labels.set_scale(self)
-        self.labels = labels
-        return self.labels
-    
-    @_docs.write_docs
-    @dims_to_spatial_axes
-    @check_input_and_output
-    def label_if(
-        self,
-        ref_image: np.ndarray | None = None,
-        filt: Callable[..., bool] | None = None,
-        *,
-        dims: Dims = None,
-        connectivity: int | None = None,
-    ) -> Label:
-        """
-        Label image if the region satisfies the condition.
-        
-        Label image using `ref_image` as reference image only if certain 
-        condition dictated in `filt` is satisfied. `regionprops_table` is called
-        inside every time image is labeled.
+        Label image using `ref_image` as reference image, or image itself. If 
+        ``filt`` is given, image will be labeled only if certain condition
+        dictated in `filt` is satisfied. `regionprops_table` is called inside
+        every time image is labeled.
         
             .. code-block:: python
             
@@ -1003,82 +1017,110 @@ class LabeledArray(MetaArray):
 
         Returns
         -------
-        LabeledArray
-            Labeled image
+        Label
+            Newly created label.
         
-        Example
-        -------
-        1. Label regions if only intensity is high.
+        Examples
+        --------
+        1. Label the image with threshold and visualize with napari.
+            >>> thr = img.threshold()
+            >>> img.label(thr)
+            >>> ip.gui.add(img)
+            
+        2. Label regions if only intensity is high.
             >>> def high_intensity(img, lbl, slice):
             >>>     return np.mean(img[slice]) > 10000
-            >>> img.label_if(lbl, filt)
+            >>> img.label(lbl, filt)
         
-        2. Label regions if no hole exists.
+        3. Label regions if no hole exists.
             >>> def no_hole(img, lbl, euler_number):
             >>>     return euler_number > 0
-            >>> img.label_if(lbl, filt)
+            >>> img.label(lbl, filt)
         
-        3. Label regions if centroids are inside themselves.
+        4. Label regions if centroids are inside themselves.
             >>> def no_hole(img, lbl, centroid):
             >>>     yc, xc = map(int, centroid)
             >>>     return lbl[yc, xc] > 0
-            >>> img.label_if(lbl, filt)
+            >>> img.label(lbl, filt)
+        
         """        
-        import pandas as pd
-        # check the shape of ref_image
+        # check the shape of label_image
         if ref_image is None:
             ref_image = self
-        
         else:
             if not isinstance(ref_image, MetaArray):
+                ref_image = np.asarray(ref_image)
                 ref_image = MetaArray(
-                    np.asarray(ref_image),
-                    axes=str(self.axes)[-self.ndim:]
+                    ref_image,
+                    axes=self.axes[-ref_image.ndim:]
                 )
-            if not self.axes.contains(ref_image.axes):
+            if not ref_image._dimension_matches(self):
                 raise ImageAxesError(
-                    "Not all the axes in `ref_image` are included in self: "
-                    f"{ref_image.axes} and {self.axes}"
+                    f"Shape mismatch. Image is {self.shape_info} but reference is"
+                    f"{ref_image.shape_info}."
                 )
-            elif not _shape_match(self, ref_image):
-                raise ImageAxesError("Shape mismatch.")
         
-        # check filter function
-        if filt is None:
-            raise ValueError("`filt` must be given.")
-        if not callable(filt):
-            raise TypeError("`filt` must be callable.")
-        
-        import inspect
-        properties = tuple(inspect.signature(filt).parameters)[2:]
-            
         c_axes = complement_axes(dims, self.axes)
         labels = largest_zeros(ref_image.shape)
-        offset = 1
-        for sl, lbl in ref_image.iter(c_axes):
-            lbl = skmes.label(lbl, background=0, connectivity=connectivity)
-            img = self.value[sl]
-            # Following lines are essentially doing the same thing as 
-            # `skmes.regionprops_table`. However, `skmes.regionprops_table`
-            # returns tuples in the separated columns in DataFrame and rename
-            # property names like "centroid-0" and "centroid-1".
-            props_obj = skmes.regionprops(lbl, img, cache=False)
-            d = {prop_name: [getattr(prop, prop_name) for prop in props_obj]
-                 for prop_name in properties}
-            df = pd.DataFrame(d)
-            del_list = [i+1 for i, r in df.iterrows() if not filt(img, lbl, **r)]
-            labels[sl] = skseg.relabel_sequential(
-                np.where(np.isin(lbl, del_list), 0, lbl),
-                offset=offset
-            )[0]
-            offset += labels.max()
         
-        labels = labels.view(Label).optimize()
+        if filt is None:
+            labels[:] = ref_image._apply_dask(
+                skmes.label, 
+                c_axes=c_axes, 
+                kwargs=dict(background=0, connectivity=connectivity)
+            ).view(np.ndarray)
+        else:
+            if not callable(filt):
+                raise TypeError("`filt` must be callable.")
+            
+            import inspect
+            import pandas as pd
+            properties = tuple(inspect.signature(filt).parameters)[2:]
+
+            offset = 1
+            for sl, lbl in ref_image.iter(c_axes):
+                lbl = skmes.label(lbl, background=0, connectivity=connectivity)
+                img = self.value[sl]
+                # Following lines are essentially doing the same thing as 
+                # `skmes.regionprops_table`. However, `skmes.regionprops_table`
+                # returns tuples in the separated columns in DataFrame and rename
+                # property names like "centroid-0" and "centroid-1".
+                props_obj = skmes.regionprops(lbl, img, cache=False)
+                d = {prop_name: [getattr(prop, prop_name) for prop in props_obj]
+                    for prop_name in properties}
+                df = pd.DataFrame(d)
+                del_list = [i + 1 for i, r in df.iterrows() if not filt(img, lbl, **r)]
+                labels[sl] = skseg.relabel_sequential(
+                    np.where(np.isin(lbl, del_list), 0, lbl),
+                    offset=offset
+                )[0]
+                offset += labels.max()
+    
+        # correct the label numbers of `labels`
+        labels = labels.view(Label)
         labels._set_info(ref_image)
+        labels = labels.increment_iter(c_axes).optimize()
         labels.set_scale(self)
         self.labels = labels
         return self.labels
     
+    @_docs.write_docs
+    @dims_to_spatial_axes
+    def label_if(
+        self,
+        ref_image: np.ndarray | None = None,
+        filt: Callable[..., bool] | None = None,
+        *,
+        dims: Dims = None,
+        connectivity: int | None = None,
+    ) -> Label:
+        warn(
+            "`label_if` is deprecated and will be removed soon. `label` method does the "
+            "same function",
+            DeprecationWarning,
+        )
+        return self.label(ref_image, filt, dims=dims, connectivity=connectivity)
+    
     @check_input_and_output
     def append_label(self, label_image: np.ndarray, new: bool = False) -> Label:
         """
@@ -1438,28 +1480,6 @@ def _iter_dict(d, nparam):
                 out[k] = v
         yield out
 
-class NotMe:
-    def __eq__(self, other):
-        return False
-
-_notme = NotMe()
-
-def _shape_match(img: LabeledArray, label: Label):
-    """
-    e.g.)
-    img   ... 12(t), 100(y), 50(x)
-    label ... 100(y), 50(x)
-        -> True
-    img   ... 12(t), 100(y), 50(x)
-    label ... 30(y), 50(x)
-        -> False
-    """    
-    img_shape = img.shape
-    label_shape = label.shape
-    return all(
-        [getattr(img_shape, str(a), _notme) == getattr(label_shape, str(a), _notme)
-         for a in label.axes]
-    )
 
 def _iter_tile_yx(ymax, xmax, imgy, imgx):
     """
