@@ -4,6 +4,8 @@ import sys
 import re
 import glob
 import itertools
+from typing import TYPE_CHECKING
+
 if sys.version_info < (3, 10):
     from typing_extensions import ParamSpec
 else:
@@ -23,6 +25,9 @@ from .arrays.bases import MetaArray
 from .arrays import ImgArray, LazyImgArray
 from ._const import Const
 
+if TYPE_CHECKING:
+    from .roi import RoiList
+
 __all__ = [
     "array", 
     "asarray", 
@@ -38,6 +43,7 @@ __all__ = [
     "imread_collection", 
     "lazy_imread", 
     "read_meta", 
+    "roiread",
     "sample_image",
     "broadcast_arrays"
 ]
@@ -98,7 +104,7 @@ def array(
         
     # Automatically determine axes
     if axes is None:
-        axes = ["x", "yx", "tyx", "tzyx", "tzcyx", "ptzcyx"][arr.ndim-1]
+        axes = ["", "x", "yx", "tyx", "tzyx", "tzcyx", "ptzcyx"][arr.ndim]
             
     self = ImgArray(arr, name=name, axes=axes)
     
@@ -757,3 +763,9 @@ def read_meta(path: str) -> dict[str]:
         "scale": image_data.scale,
         "metadata": image_data.metadata
     }
+
+def roiread(path: str) -> RoiList:
+    """Read a Roi.zip file as a RoiList object."""
+    from .roi import RoiList
+    
+    return RoiList.fromfile(path)
\ No newline at end of file
